''**************************************
''
''  WIZnet Test Driver Ver. 00.2
''
''  Timothy D. Swieter, E.I.
''  company name to go here
''
''  Updated: December 17, 2007
''
''Description:
''This program is used to test the driver for WIZnet W5100  
''
''reference:
''      W5100_Datasheet_v1_1_2.pdf (Datasheet for chip)
''      2711015Eady.pdf (Circuit Cellar Magazine Article)
''      http://bbs.circuitcellar.com/phpBB2/viewforum.php?f=30
''
''**************************************

CON
'***************************************
'  Hardware related settings
'***************************************
  _clkmode = xtal1 + pll16x                             'Use the PLL to multiple the external clock by 16
  _xinfreq = 5_000_000                                  'An external clock of 5MHz. is used (80MHz. operation)

'***************************************
'  System Definitions      
'***************************************

  _OUTPUT       = 1             'Sets pin to output in DIRA register
  _INPUT        = 0             'Sets pin to input in DIRA register  
  _HIGH         = 1             'High=ON=1=3.3v DC
  _ON           = 1
  _LOW          = 0             'Low=OFF=0=0v DC
  _OFF          = 0
  _ENABLE       = 1             'Enable (turn on) function/mode
  _DISABLE      = 0             'Disable (turn off) function/mode
  
  'Used for debug string construction, remove when debug remove
  CR            = 13                                    'Go to next line?                      
  FF            = 12                                    'Clear screen
  LF            = 10                                    'Go to next line
  
VAR               'Variables to be located here
'None at this time

OBJ               'Object declaration to be located here

  ETHERNET      : "TIM_WIZnet_Driver_Ver002.spin"
  Debug         : "PC_Debug.spin"                       'This object is used for debugging purposes only

PUB MAIN | temp0, temp1, temp2, temp3, temp4, temp5, testlong, testbit


  ' Setup the Debug parameters so items can be viewed in PropTerminal
  debug.start(460_800)                                  'start terminal
  PauseMSec(5000)                                       'Wait 5 second for me to switch applications to hyperterminal
  debug.str(string(FF, "Ethernet Tester", CR, LF, LF))  'print string

  'These are the LED Outputs of the demo board
  dira[16] := _OUTPUT
  dira[17] := _OUTPUT
  dira[18] := _OUTPUT
  dira[19] := _OUTPUT
  dira[20] := _OUTPUT
  dira[21] := _OUTPUT
  dira[22] := _OUTPUT
  dira[23] := _OUTPUT

  'Setup the module for being used
  '                   ss, sclk, mosi, miso
  ETHERNET.InitWIZnet(0, 1, 2, 3)
  debug.str(string("Init", CR, LF))                     'print string
  PauseMSec(200)
  
  'software reset of the module
  ETHERNET.ResetWIZnet
  debug.str(string("Reset", CR, LF))                    'print string
  PauseMSec(1000)

  'configure the addressess
  ETHERNET.SetGatewayAddress(192,168,0,1)
  ETHERNET.SetMACAddress($10,$00,$00,$00,$00,$01)
  ETHERNET.SetSubnetMask(255,255,255,0)
  ETHERNET.SetIPAddress(192,168,0,32)
  debug.str(string("Address Set", CR, LF))              'print string

  ETHERNET.PingBlockDisable

  PauseMSec(1000)                                       'Wait 5 second

  temp0 := ETHERNET.ReadIPAddress(0)
  temp1 := ETHERNET.ReadIPAddress(1)
  temp2 := ETHERNET.ReadIPAddress(2)
  temp3 := ETHERNET.ReadIPAddress(3)
  debug.str(string("IP Address: ")) 'print string
  debug.dec(temp0)
  debug.str(string("."))
  debug.dec(temp1)
  debug.str(string("."))
  debug.dec(temp2)
  debug.str(string("."))
  debug.dec(temp3)
  debug.str(string(CR, LF))
  PauseMSec(250)
   
  temp0 := ETHERNET.ReadMACAddress(0)
  temp1 := ETHERNET.ReadMACAddress(1)
  temp2 := ETHERNET.ReadMACAddress(2)
  temp3 := ETHERNET.ReadMACAddress(3)
  temp4 := ETHERNET.ReadMACAddress(4)
  temp5 := ETHERNET.ReadMACAddress(5)
   
  debug.str(string("MAC Address: ")) 'print string
  debug.hex(temp0,2)
  debug.str(string("."))
  debug.hex(temp1,2)
  debug.str(string("."))
  debug.hex(temp2,2)
  debug.str(string("."))
  debug.hex(temp3,2)
  debug.str(string("."))
  debug.hex(temp4,2)
  debug.str(string("."))
  debug.hex(temp5,2)
  debug.str(string(CR, LF))
  PauseMSec(250)
   
  temp0 := ETHERNET.ReadGatewayAddress(0)
  temp1 := ETHERNET.ReadGatewayAddress(1)
  temp2 := ETHERNET.ReadGatewayAddress(2)
  temp3 := ETHERNET.ReadGatewayAddress(3)
   
  debug.str(string("Gateway Address: ")) 'print string
  debug.dec(temp0)
  debug.str(string("."))
  debug.dec(temp1)
  debug.str(string("."))
  debug.dec(temp2)
  debug.str(string("."))
  debug.dec(temp3)
  debug.str(string(CR, LF))
  PauseMSec(250)
   
  temp0 := ETHERNET.ReadSubnetMask(0)
  temp1 := ETHERNET.ReadSubnetMask(1)
  temp2 := ETHERNET.ReadSubnetMask(2)
  temp3 := ETHERNET.ReadSubnetMask(3)
   
  debug.str(string("Subnet Mask: ")) 'print string
  debug.dec(temp0)
  debug.str(string("."))
  debug.dec(temp1)
  debug.str(string("."))
  debug.dec(temp2)
  debug.str(string("."))
  debug.dec(temp3)
  debug.str(string(CR, LF))
  PauseMSec(250)

  ETHERNET.InitSocket

  PauseMSec(1000)
  
  TestReadWrite
  repeat

PUB TestReadWrite | temp0, temp1, hibyte, lobyte, getsize, getoffset, getstartaddress, addri, header_size, DestAddr[8], ipdata[200], i, udpdatasize

 repeat

  temp0 := 0

  repeat while temp0 == 0
    temp0 := ETHERNET.ReadWizReg(ETHERNET#_S0_IR)
    temp0&= %00000100
    debug.str(string(FF, "Waiting...", CR, LF)) 'print string
    debug.bin(temp0,8)          

  ETHERNET.WriteWizReg(ETHERNET#_S0_IR, ETHERNET#_RECVM)

  hibyte := ETHERNET.ReadWizReg(ETHERNET#_S0_RX_RSR0)
  lobyte := ETHERNET.ReadWizReg(ETHERNET#_S0_RX_RSR1)
  getsize := (hibyte << 8) | lobyte

  debug.str(string("UDP Size: ")) 'print string
  debug.str(string(CR, LF))                  
  debug.dec(hibyte)
  debug.str(string(CR, LF))                  
  debug.dec(lobyte)
  debug.str(string(CR, LF))                  
  debug.dec(getsize)
  debug.str(string(CR, LF))

  hibyte := ETHERNET.ReadWizReg(ETHERNET#_S0_RX_RD0)
  lobyte := ETHERNET.ReadWizReg(ETHERNET#_S0_RX_RD1)
  getoffset := ((hibyte << 8) | lobyte) & ETHERNET#_gS0_RX_MASK

  debug.str(string("Offset: ")) 'print string
  debug.str(string(CR, LF))                  
  debug.dec(hibyte)
  debug.str(string(CR, LF))                  
  debug.dec(lobyte)
  debug.str(string(CR, LF))                  
  debug.dec(getoffset)
  debug.str(string(CR, LF))

  getstartaddress := ETHERNET#_gS0_RX_BASE + getoffset

  header_size := 8
  addri := getstartaddress

  repeat i from 0 to (header_size-1)
    DestAddr[i] := ETHERNET.ReadWizReg(addri)
    addri++
    getoffset++
    
  getstartaddress := ETHERNET#_gS0_RX_BASE + getoffset
  udpdatasize := getsize - header_size
  addri := getstartaddress

  repeat i from 0 to (udpdatasize-1)
    ipdata[i] := ETHERNET.ReadWizReg(addri)
    addri++
    getoffset++


  hibyte := ETHERNET.ReadWizReg(ETHERNET#_S0_RX_RD0)
  lobyte := ETHERNET.ReadWizReg(ETHERNET#_S0_RX_RD1)
  temp1:= ((hibyte << 8) | lobyte) & ETHERNET#_gS0_RX_MASK
  temp1 := temp1 + getsize + header_size
  lobyte := getoffset & $00FF
  hibyte := (getoffset >> 8) & $00FF

  ETHERNET.WriteWizReg(ETHERNET#_S0_RX_RD0,hibyte)
  ETHERNET.WriteWizReg(ETHERNET#_S0_RX_RD1,lobyte)
  ETHERNET.WriteWizReg(ETHERNET#_S0_CR, ETHERNET#_RECV)

  if ipdata[0] == "L"
    if lookdown(ipdata[1]: "9".."0")
      case ipdata[1]

        "0": temp1 := 0 <<16
        "1": temp1 := 1 <<16
        "2": temp1 := 2 <<16
        "3": temp1 := 4 <<16
        "4": temp1 := 8 <<16
        "5": temp1 := 16 <<16
        "6": temp1 := 32 <<16
        "7": temp1 := 64 <<16
        "8": temp1 := 128 <<16
        "9": temp1 := 255 <<16
        
'   outa[16] := 1
'   temp1 := ipdata[1] << 16
'   temp1 := 128 << 16
    outa := temp1
  else
    outa := 0
'   outa[16] := 0
  
  'done receiving info, now send it

  addri := ETHERNET#_S0_DIPR0
  repeat i from 0 to 3
    ETHERNET.WriteWizReg(addri, DestAddr[i])
    addri++

  addri := ETHERNET#_S0_DPORT0
  repeat i from 0 to 1
    ETHERNET.WriteWizReg(addri, DestAddr[4+i])
    addri++

  hibyte := ETHERNET.ReadWizReg(ETHERNET#_S0_TX_WR0)
  lobyte := ETHERNET.ReadWizReg(ETHERNET#_S0_TX_WR1)
  getoffset := ((hibyte << 8) | lobyte) & ETHERNET#_gS0_TX_MASK

  getstartaddress := ETHERNET#_gS0_TX_Base + getoffset
  addri := getstartaddress

  repeat i from 0 to (udpdatasize - 1)
    ETHERNET.WriteWizReg(addri,ipdata[i])
    addri++
    getoffset++

  lobyte := getoffset & $00FF
  hibyte := (getoffset >> 8) & $00FF
    
  ETHERNET.WriteWizReg(ETHERNET#_S0_TX_WR0, hibyte)
  ETHERNET.WriteWizReg(ETHERNET#_S0_TX_WR1, lobyte)
  ETHERNET.WriteWizReg(ETHERNET#_S0_CR, ETHERNET#_SEND)

PRI PauseMSec(Duration)
''Pause execution in milliseconds.
''Duration = number of milliseconds to delay
  
  waitcnt(((clkfreq / 1_000 * Duration - 3932) #> 381) + cnt)